@@ -57,6 +57,8 @@
 #ifndef TRSM_NB
    #ifdef TREAL
       #define TRSM_NB    NB
+   #else
+      #define TRSM_NB     4
    #endif
 #endif
 
@@ -103,7 +105,6 @@ void Mjoin( PATL, trsm )
    TYPE                       alpha0 = (TYPE)(ALPHA);
    const TYPE                 negone = ATL_rnone, one = ATL_rone;
 #else
-   int TRSM_NB=1;
    TYPE                       negone[2] = { ATL_rnone, ATL_rzero },
                               one   [2] = { ATL_rone,  ATL_rzero };
 #endif
@@ -129,9 +130,6 @@ void Mjoin( PATL, trsm )
 
    if( SIDE == AtlasLeft )
    {
-      #ifdef TCPLX
-         TRSM_NB = 4;
-      #endif
       if( TRANS == AtlasNoTrans )
       {
          type.Tgemm = Mjoin( PATL, gemmNN_RB );
