@@ -1,6 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.3.15
- **************** THIS IS AN UNSUPPORTED DEVELOPER RELEASE *****************
+ *             Automatically Tuned Linear Algebra Software v3.4.0
  *                   (C) Copyright 2000 Peter Soendergaard
  *
  * Redistribution and use in source and binary forms, with or without
@@ -18,8 +17,8 @@
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
@@ -53,7 +52,7 @@ void ATL_USERMM
    /*  align_jumps = 0  */
    /*  prec = 'single'  */
    /*  m_cleanup = {}  */
-   /*  used_outside_len = 60  */
+   /*  used_outside_len = 64  */
    /*  k_cleanup = {'mustfit': 56, 'method': 'acc1', 'compile': 1}  */
    /*  outputdir = 'Linux_PIII/'  */
    /*  arch = 'sse'  */
@@ -61,7 +60,7 @@ void ATL_USERMM
    /*  atlasname = 'Linux_PIII'  */
    /*  method = 'acc'  */
    /*  used_lastuse = 'b'  */
-   /*  outside_len = 60  */
+   /*  outside_len = 64  */
    /*  lastuse = 1  */
    /*  veclen = 4  */
    /*  sched = ['fuse', 'spread']  */
@@ -79,8 +78,8 @@ void ATL_USERMM
    float *pC0 = C;
    const float *stM = A + MB*KB;
    const float *stN = B + N*KB;
-   const int incAm = 2*KB-KB+60;
-   const int incBm = -KB+60;
+   const int incAm = 2*KB-KB+64;
+   const int incBm = -KB+64;
    const int incCm = (2 SHIFT);
    const int incAn = -MB*KB;
    const int incBn = 1*KB;
@@ -193,11 +192,17 @@ void ATL_USERMM
          vec_add_rr(reg3,reg0);
          vec_mul_mr_a(pA0+52+KB,reg6);
          vec_add_rr(reg6,reg1);
-         vec_mov_mr_a(pB0+56,reg5);
-         vec_mov_mr_a(pA0+56,reg2);
+         vec_mov_mr_a(pB0+56,reg7);
+         vec_mov_mr_a(pA0+56,reg4);
+         vec_mul_rr(reg7,reg4);
+         vec_add_rr(reg4,reg0);
+         vec_mul_mr_a(pA0+56+KB,reg7);
+         vec_add_rr(reg7,reg1);
+         vec_mov_mr_a(pB0+60,reg5);
+         vec_mov_mr_a(pA0+60,reg2);
          vec_mul_rr(reg5,reg2);
          vec_add_rr(reg2,reg0);
-         vec_mul_mr_a(pA0+56+KB,reg5);
+         vec_mul_mr_a(pA0+60+KB,reg5);
          vec_add_rr(reg5,reg1);
          vec_sum(reg0);
          vec_sum(reg1);
