@@ -1,6 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.3.15
- **************** THIS IS AN UNSUPPORTED DEVELOPER RELEASE *****************
+ *             Automatically Tuned Linear Algebra Software v3.4.0
  *                    (C) Copyright 1999 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -18,8 +17,8 @@
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
@@ -418,21 +417,22 @@ static void gemvNle4(const int M, const int N, const TYPE *A, const int lda,
    }
 }
 
-static void gemv16x2(const int M, const int N, const TYPE *A, const int lda,
+static void gemv16x4(const int M, const int N, const TYPE *A, const int lda,
                      const TYPE *X, const SCALAR beta, TYPE *Y)
 /*
- * 16x2 with feeble prefetch
+ * 16x4 with feeble prefetch
  */
 {
-   const int M16 = (M>>4)<<4, N2 = (N>>1)<<1, nr = N-N2;
-   const int incA = lda << 1, incAm = 16 - N2*lda;
-   const TYPE *stX = X + N2 - 2, *x;
-   const TYPE *A0 = A, *A1 = A + lda;
+   int j;
+   const int M16 = (M>>4)<<4, N4 = (N>>2)<<2, nr = N-N4+4;
+   const int incA = lda << 2, incAm = 16 - (N4-4)*lda, incAm0 = 16 - N*lda;
+   const TYPE *stX = X + N4, *x;
+   const TYPE *A0 = A, *A1 = A + lda, *A2 = A1 + lda, *A3 = A2 + lda;
    TYPE *stY = Y + M16;
-   register TYPE x0, x1;
+   register TYPE x0, x1, x2, x3;
    register TYPE y0, y1, y2, y3, y4, y5, y6, y7;
    register TYPE y8, y9, y10, y11, y12, y13, y14, y15;
-   register TYPE p0, p1;
+   register TYPE p0, p1, p2, p3;
 
    if (N > 4)
    {
@@ -446,166 +446,134 @@ static void gemv16x2(const int M, const int N, const TYPE *A, const int lda,
             #elif defined BETAX
                x0 = beta;
                y0 = *Y; y1 = Y[1]; y2 = Y[2];  y3 = Y[3];
-               y4 = Y[4]; y5 = Y[5]; y6 = Y[6];  y7 = Y[7];
                y8 = Y[8]; y9 = Y[9]; y10 = Y[10];  y11 = Y[11];
+               y4 = Y[4]; y5 = Y[5]; y6 = Y[6];  y7 = Y[7];
                y12 = Y[12]; y13 = Y[13]; y14 = Y[14];  y15 = Y[15];
                y0 *= x0; y1 *= x0; y2 *= x0; y3 *= x0;
-               y4 *= x0; y5 *= x0; y6 *= x0; y7 *= x0;
                y8 *= x0; y9 *= x0; y10 *= x0; y11 *= x0;
+               y4 *= x0; y5 *= x0; y6 *= x0; y7 *= x0;
                y12 *= x0; y13 *= x0; y14 *= x0; y15 *= x0;
             #else
                y0 = *Y; y1 = Y[1]; y2 = Y[2];  y3 = Y[3];
-               y4 = Y[4]; y5 = Y[5]; y6 = Y[6];  y7 = Y[7];
                y8 = Y[8]; y9 = Y[9]; y10 = Y[10];  y11 = Y[11];
+               y4 = Y[4]; y5 = Y[5]; y6 = Y[6];  y7 = Y[7];
                y12 = Y[12]; y13 = Y[13]; y14 = Y[14];  y15 = Y[15];
             #endif
-            p0 = *A0;
-            p1 = A1[1];
-            x = X;
-            do
+            p0 = *A0; p1 = A1[1];
+            p2 = A2[2]; p3 = A3[3];
+            x0 = *X; x1 = X[1]; x2 = X[2]; x3 = X[3];
+            x = X + 4;
+            if (N4 != 4)
             {
-               x0 = *x; x1 = x[1]; x += 2;
-               y0 += x0 * p0; p0 = A0[incA];
-               y1 += x1 * p1; p1 = A1[incA+1];
-               y8 += x0 * A0[8];
-               y9 += x1 * A1[9];
-               y4 += x0 * A0[4];
-               y5 += x1 * A1[5];
-               y12 += x0 * A0[12];
-               y13 += x1 * A1[13];
-               y2 += x0 * A0[2];
-               y3 += x1 * A1[3];
-               y10 += x0 * A0[10];
-               y11 += x1 * A1[11];
-               y6 += x0 * A0[6];
-               y7 += x1 * A1[7];
-               y14 += x0 * A0[14];
-               y15 += x1 * A1[15];
+               do
+               {
+                  y0  += x0 * p0; p0 = A0[incA];
+                  y1  += x1 * p1; p1 = A1[incA+1];
+                  y2  += x2 * p2; p2 = A2[incA+2];
+                  y3  += x3 * p3; p3 = A3[incA+3];
+                  y8  += x0 * A0[8];
+                  y9  += x1 * A1[9];
+                  y10 += x2 * A2[10];
+                  y11 += x3 * A3[11];
+                  y4  += x0 * A0[4];
+                  y5  += x1 * A1[5];
+                  y6  += x2 * A2[6];
+                  y7  += x3 * A3[7];
+                  y12 += x0 * A0[12];
+                  y13 += x1 * A1[13];
+                  y14 += x2 * A2[14];
+                  y15 += x3 * A3[15];
 
-               y0  += x1 * *A1;
-               y1  += x0 * A0[1];
-               y8  += x1 * A1[8];
-               y9  += x0 * A0[9];
-               y4  += x1 * A1[4];
-               y5  += x0 * A0[5];
-               y12 += x1 * A1[12];
-               y13 += x0 * A0[13];
-               y2  += x1 * A1[2];
-               y3  += x0 * A0[3];
-               y10 += x1 * A1[10];
-               y11 += x0 * A0[11];
-               y6  += x1 * A1[6];
-               y7  += x0 * A0[7];
-               y14 += x1 * A1[14]; A1 += incA;
-               y15 += x0 * A0[15]; A0 += incA;
+                  y0  += x1 * *A1;
+                  y1  += x0 * A0[1];
+                  y2  += x0 * A0[2];
+                  y3  += x0 * A0[3];
+                  y8  += x1 * A1[8];
+                  y9  += x0 * A0[9];
+                  y10 += x0 * A0[10];
+                  y11 += x0 * A0[11];
+                  y4  += x1 * A1[4];
+                  y5  += x0 * A0[5];
+                  y6  += x0 * A0[6];
+                  y7  += x0 * A0[7];
+                  y12 += x1 * A1[12];
+                  y13 += x0 * A0[13];
+                  y14 += x0 * A0[14];
+                  y15 += x0 * A0[15]; x0 = *x;
+
+                  y0  += x2 * *A2;
+                  y1  += x2 * A2[1]; A0 += incA;
+                  y2  += x1 * A1[2];
+                  y3  += x1 * A1[3];
+                  y8  += x2 * A2[8];
+                  y9  += x2 * A2[9];
+                  y10 += x1 * A1[10];
+                  y11 += x1 * A1[11];
+                  y4  += x2 * A2[4];
+                  y5  += x2 * A2[5];
+                  y6  += x1 * A1[6];
+                  y7  += x1 * A1[7];
+                  y12 += x2 * A2[12];
+                  y13 += x2 * A2[13];
+                  y14 += x1 * A1[14];
+                  y15 += x1 * A1[15]; x1 = x[1];
+
+                  y0  += x3 * *A3;
+                  y1  += x3 * A3[1];
+                  y2  += x3 * A3[2]; A1 += incA;
+                  y3  += x2 * A2[3];
+                  y8  += x3 * A3[8];
+                  y9  += x3 * A3[9];
+                  y10 += x3 * A3[10];
+                  y11 += x2 * A2[11];
+                  y4  += x3 * A3[4];
+                  y5  += x3 * A3[5];
+                  y6  += x3 * A3[6];
+                  y7  += x2 * A2[7];
+                  y12 += x3 * A3[12];
+                  y13 += x3 * A3[13];
+                  y14 += x3 * A3[14]; x3 = x[3]; A3 += incA;
+                  y15 += x2 * A2[15]; x2 = x[2]; x += 4; A2 += incA;
+               }
+               while (x != stX);
             }
-            while (x != stX);
-            if (!nr) /* 2 cols left */
+            x -= 4;
+            for (j=0; j != nr; j++, A0 += lda)
             {
-               x0 = *x; x1 = x[1]; x += 2;
-               y0 += x0 * p0;
-               y1 += x1 * p1;
-               y8 += x0 * A0[8];
-               y9 += x1 * A1[9];
-               y4 += x0 * A0[4];
-               y5 += x1 * A1[5];
-               y12 += x0 * A0[12];
-               y13 += x1 * A1[13];
-               y2 += x0 * A0[2];
-               y3 += x1 * A1[3];
-               y10 += x0 * A0[10];
-               y11 += x1 * A1[11];
-               y6 += x0 * A0[6];
-               y7 += x1 * A1[7];
-               y14 += x0 * A0[14];
-               y15 += x1 * A1[15];
-
-               y0  += x1 * *A1;
-               y1  += x0 * A0[1];
-               y8  += x1 * A1[8];
-               y9  += x0 * A0[9];
-               y4  += x1 * A1[4];
-               y5  += x0 * A0[5];
-               y12 += x1 * A1[12];
-               y13 += x0 * A0[13];
-               y2  += x1 * A1[2];
-               y3  += x0 * A0[3];
-               y10 += x1 * A1[10];
-               y11 += x0 * A0[11];
-               y6  += x1 * A1[6];
-               y7  += x0 * A0[7];
-               y14 += x1 * A1[14]; A1 += incA;
-               y15 += x0 * A0[15]; A0 += incA;
-            }
-            else     /* 3 cols left */
-            {
-               x0 = *x; x1 = x[1]; x += 2;
-               y0 += x0 * p0; p0 = A0[incA];
-               y1 += x1 * p1;
-               y8 += x0 * A0[8];
-               y9 += x1 * A1[9];
-               y4 += x0 * A0[4];
-               y5 += x1 * A1[5];
-               y12 += x0 * A0[12];
-               y13 += x1 * A1[13];
-               y2 += x0 * A0[2];
-               y3 += x1 * A1[3];
-               y10 += x0 * A0[10];
-               y11 += x1 * A1[11];
-               y6 += x0 * A0[6];
-               y7 += x1 * A1[7];
-               y14 += x0 * A0[14];
-               y15 += x1 * A1[15];
-
-               y0  += x1 * *A1;
-               y1  += x0 * A0[1];
-               y8  += x1 * A1[8];
-               y9  += x0 * A0[9];
-               y4  += x1 * A1[4];
-               y5  += x0 * A0[5];
-               y12 += x1 * A1[12];
-               y13 += x0 * A0[13];
-               y2  += x1 * A1[2];
-               y3  += x0 * A0[3];
-               y10 += x1 * A1[10];
-               y11 += x0 * A0[11];
-               y6  += x1 * A1[6];
-               y7  += x0 * A0[7];
-               y14 += x1 * A1[14]; A1 += incA;
-               y15 += x0 * A0[15]; A0 += incA;
-
-               x0 = *x;
+               x0 = x[j];
                y0  += x0 * *A0;
                y1  += x0 * A0[1];
                y2  += x0 * A0[2];
                y3  += x0 * A0[3];
-               y4  += x0 * A0[4];
-               y5  += x0 * A0[5];
-               y6  += x0 * A0[6];
-               y7  += x0 * A0[7];
                y8  += x0 * A0[8];
                y9  += x0 * A0[9];
                y10 += x0 * A0[10];
                y11 += x0 * A0[11];
+               y4  += x0 * A0[4];
+               y5  += x0 * A0[5];
+               y6  += x0 * A0[6];
+               y7  += x0 * A0[7];
                y12 += x0 * A0[12];
                y13 += x0 * A0[13];
                y14 += x0 * A0[14];
                y15 += x0 * A0[15];
             }
-            *Y = y0;
-            A0 += incAm;
-            Y[1] = y1;
+            A0 += incAm0;
+            *Y   = y0;
+            Y[ 1] = y1 ;
+            Y[ 2] = y2 ;
+            Y[ 3] = y3 ;
             A1 += incAm;
-            Y[2] = y2;
-            Y[3] = y3;
-            Y[4] = y4;
-            Y[5] = y5;
-            Y[6] = y6;
-            Y[7] = y7;
-            Y[8] = y8;
-            Y[9] = y9;
+            Y[ 8] = y8 ;
+            Y[ 9] = y9 ;
             Y[10] = y10;
             Y[11] = y11;
+            A2 += incAm;
+            Y[ 4] = y4 ;
+            Y[ 5] = y5 ;
+            Y[ 6] = y6 ;
+            Y[ 7] = y7 ;
+            A3 += incAm;
             Y[12] = y12;
             Y[13] = y13;
             Y[14] = y14;
@@ -622,5 +590,5 @@ void Mjoin(Mjoin(Mjoin(Mjoin(Mjoin(PATL,gemvN),NM),_x1),BNM),_y1)
    (const int M, const int N, const SCALAR alpha, const TYPE *A, const int lda,
     const TYPE *X, const int incX, const SCALAR beta, TYPE *Y, const int incY)
 {
-   gemv16x2(M, N, A, lda, X, beta, Y);
+   gemv16x4(M, N, A, lda, X, beta, Y);
 }
