@@ -130,7 +130,6 @@ int ATL_UIAMAX(const int N, const TYPE *X, const int incX)
    int cwrd = ATL_MulBySize(N)>>4;
    int im=0;
    int n0, n1, n2;
-   size_t ii;
 
    if (N > 64)
    {
@@ -149,7 +148,7 @@ int ATL_UIAMAX(const int N, const TYPE *X, const int incX)
       }
       else cwrd = ATL_GetCtrl(64, (cwrd+3)>>2, 4);
       ATL_pfavR(X, cwrd, 0);
-      for (n0=0, ii=(size_t)X; n0<N && (ii>>4)<<4 != ii; n0++, ii+=ATL_sizeof);
+      n0 = ATL_AlignOffset(N, X, ATL_sizeof, ATL_MulBySize(4));
       if (n0) im = ATL_amax(n0, X, incX);
       n1 = ((N - n0)>>2)<<2;
       n2 = N - n0 - n1;
