@@ -821,7 +821,7 @@ void GenMainRout(char pre, int n, int *ix, int *iy, int *ia, int *ib,
       fprintf(fpout,
          "%s{\n", spc);
       fprintf(fpout,
-         "%s   Mjoin(PATLU,axpy)(N<<1, *alpha, X, 1, Y, 1);\n", spc);
+         "%s   Mjoin(PATLU,axpy)(N+N, *alpha, X, 1, Y, 1);\n", spc);
       fprintf(fpout,
          "%s   return;\n", spc);
       fprintf(fpout,
