@@ -704,7 +704,7 @@ void GenMainRout(char pre, int n, int *ix, int *iy, int *ia, int *ib,
       fprintf(fpout,
          "%s{\n", spc);
       fprintf(fpout,
-         "%s   Mjoin(PATLU,set)(N<<1, *alpha, X, 1);\n", spc);
+         "%s   Mjoin(PATLU,set)(N+N, *alpha, X, 1);\n", spc);
       fprintf(fpout,
          "%s   return;\n", spc);
       fprintf(fpout,
